"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .billingcadence import BillingCadence
from .billingmodel import BillingModel
from .billingperiod import BillingPeriod
from .billingtier import BillingTier
from .dto_createpricetier import DtoCreatePriceTier, DtoCreatePriceTierTypedDict
from .dto_priceunitconfig import DtoPriceUnitConfig, DtoPriceUnitConfigTypedDict
from .invoicecadence import InvoiceCadence
from .price_transformquantity import (
    PriceTransformQuantity,
    PriceTransformQuantityTypedDict,
)
from .pricetype import PriceType
from .priceunittype import PriceUnitType
from flexprice.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, List, Optional
from typing_extensions import NotRequired, TypedDict


class DtoSubscriptionPriceCreateRequestTypedDict(TypedDict):
    billing_cadence: BillingCadence
    billing_model: BillingModel
    billing_period: BillingPeriod
    invoice_cadence: InvoiceCadence
    price_unit_type: PriceUnitType
    type: PriceType
    amount: NotRequired[str]
    billing_period_count: NotRequired[int]
    description: NotRequired[str]
    display_name: NotRequired[str]
    end_date: NotRequired[str]
    filter_values: NotRequired[Dict[str, List[str]]]
    lookup_key: NotRequired[str]
    metadata: NotRequired[Dict[str, str]]
    meter_id: NotRequired[str]
    min_quantity: NotRequired[int]
    price_unit_config: NotRequired[DtoPriceUnitConfigTypedDict]
    start_date: NotRequired[str]
    tier_mode: NotRequired[BillingTier]
    tiers: NotRequired[List[DtoCreatePriceTierTypedDict]]
    transform_quantity: NotRequired[PriceTransformQuantityTypedDict]
    trial_period: NotRequired[int]


class DtoSubscriptionPriceCreateRequest(BaseModel):
    billing_cadence: BillingCadence

    billing_model: BillingModel

    billing_period: BillingPeriod

    invoice_cadence: InvoiceCadence

    price_unit_type: PriceUnitType

    type: PriceType

    amount: Optional[str] = None

    billing_period_count: Optional[int] = None

    description: Optional[str] = None

    display_name: Optional[str] = None

    end_date: Optional[str] = None

    filter_values: Optional[Dict[str, List[str]]] = None

    lookup_key: Optional[str] = None

    metadata: Optional[Dict[str, str]] = None

    meter_id: Optional[str] = None

    min_quantity: Optional[int] = None

    price_unit_config: Optional[DtoPriceUnitConfig] = None

    start_date: Optional[str] = None

    tier_mode: Optional[BillingTier] = None

    tiers: Optional[List[DtoCreatePriceTier]] = None

    transform_quantity: Optional[PriceTransformQuantity] = None

    trial_period: Optional[int] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "amount",
                "billing_period_count",
                "description",
                "display_name",
                "end_date",
                "filter_values",
                "lookup_key",
                "metadata",
                "meter_id",
                "min_quantity",
                "price_unit_config",
                "start_date",
                "tier_mode",
                "tiers",
                "transform_quantity",
                "trial_period",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from flexprice import models, utils
from flexprice._hooks import HookContext
from flexprice.types import OptionalNullable, UNSET
from flexprice.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Dict, Mapping, Optional


class EntityIntegrationMappings(BaseSDK):
    def create_entity_integration_mapping(
        self,
        *,
        entity_id: str,
        entity_type: models.IntegrationEntityType,
        provider_entity_id: str,
        provider_type: str,
        metadata: Optional[Dict[str, Any]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoEntityIntegrationMappingResponse:
        r"""Create entity integration mapping

        Use when linking a FlexPrice entity to an external system (e.g. CRM or payment provider) so you can sync or reconcile by external ID.

        :param entity_id:
        :param entity_type:
        :param provider_entity_id:
        :param provider_type:
        :param metadata:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.DtoCreateEntityIntegrationMappingRequest(
            entity_id=entity_id,
            entity_type=entity_type,
            metadata=metadata,
            provider_entity_id=provider_entity_id,
            provider_type=provider_type,
        )

        req = self._build_request(
            method="POST",
            path="/entity-integration-mappings",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.DtoCreateEntityIntegrationMappingRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createEntityIntegrationMapping",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "409", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(
                models.DtoEntityIntegrationMappingResponse, http_res
            )
        if utils.match_response(http_res, ["400", "401", "409"], "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )

    async def create_entity_integration_mapping_async(
        self,
        *,
        entity_id: str,
        entity_type: models.IntegrationEntityType,
        provider_entity_id: str,
        provider_type: str,
        metadata: Optional[Dict[str, Any]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoEntityIntegrationMappingResponse:
        r"""Create entity integration mapping

        Use when linking a FlexPrice entity to an external system (e.g. CRM or payment provider) so you can sync or reconcile by external ID.

        :param entity_id:
        :param entity_type:
        :param provider_entity_id:
        :param provider_type:
        :param metadata:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.DtoCreateEntityIntegrationMappingRequest(
            entity_id=entity_id,
            entity_type=entity_type,
            metadata=metadata,
            provider_entity_id=provider_entity_id,
            provider_type=provider_type,
        )

        req = self._build_request_async(
            method="POST",
            path="/entity-integration-mappings",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.DtoCreateEntityIntegrationMappingRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createEntityIntegrationMapping",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "409", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(
                models.DtoEntityIntegrationMappingResponse, http_res
            )
        if utils.match_response(http_res, ["400", "401", "409"], "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )

    def delete_entity_integration_mapping(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ):
        r"""Delete entity integration mapping

        Use when unlinking a FlexPrice entity from an external system or cleaning up stale integration mappings.

        :param id: Entity integration mapping ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.DeleteEntityIntegrationMappingRequest(
            id=id,
        )

        req = self._build_request(
            method="DELETE",
            path="/entity-integration-mappings/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="deleteEntityIntegrationMapping",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "204", "*"):
            return
        if utils.match_response(http_res, ["400", "401", "404"], "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )

    async def delete_entity_integration_mapping_async(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ):
        r"""Delete entity integration mapping

        Use when unlinking a FlexPrice entity from an external system or cleaning up stale integration mappings.

        :param id: Entity integration mapping ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.DeleteEntityIntegrationMappingRequest(
            id=id,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/entity-integration-mappings/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="deleteEntityIntegrationMapping",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "401", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "204", "*"):
            return
        if utils.match_response(http_res, ["400", "401", "404"], "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .dto_paymentattemptresponse import (
    DtoPaymentAttemptResponse,
    DtoPaymentAttemptResponseTypedDict,
)
from .paymentdestinationtype import PaymentDestinationType
from .paymentmethodtype import PaymentMethodType
from .paymentstatus import PaymentStatus
from flexprice.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, List, Optional
from typing_extensions import NotRequired, TypedDict


class DtoPaymentResponseTypedDict(TypedDict):
    amount: NotRequired[str]
    attempts: NotRequired[List[DtoPaymentAttemptResponseTypedDict]]
    created_at: NotRequired[str]
    created_by: NotRequired[str]
    currency: NotRequired[str]
    destination_id: NotRequired[str]
    destination_type: NotRequired[PaymentDestinationType]
    error_message: NotRequired[str]
    failed_at: NotRequired[str]
    gateway_metadata: NotRequired[Dict[str, str]]
    gateway_payment_id: NotRequired[str]
    gateway_tracking_id: NotRequired[str]
    id: NotRequired[str]
    idempotency_key: NotRequired[str]
    invoice_number: NotRequired[str]
    metadata: NotRequired[Dict[str, str]]
    payment_gateway: NotRequired[str]
    payment_method_id: NotRequired[str]
    payment_method_type: NotRequired[PaymentMethodType]
    payment_status: NotRequired[PaymentStatus]
    payment_url: NotRequired[str]
    refunded_at: NotRequired[str]
    save_card_and_make_default: NotRequired[bool]
    succeeded_at: NotRequired[str]
    tenant_id: NotRequired[str]
    track_attempts: NotRequired[bool]
    updated_at: NotRequired[str]
    updated_by: NotRequired[str]


class DtoPaymentResponse(BaseModel):
    amount: Optional[str] = None

    attempts: Optional[List[DtoPaymentAttemptResponse]] = None

    created_at: Optional[str] = None

    created_by: Optional[str] = None

    currency: Optional[str] = None

    destination_id: Optional[str] = None

    destination_type: Optional[PaymentDestinationType] = None

    error_message: Optional[str] = None

    failed_at: Optional[str] = None

    gateway_metadata: Optional[Dict[str, str]] = None

    gateway_payment_id: Optional[str] = None

    gateway_tracking_id: Optional[str] = None

    id: Optional[str] = None

    idempotency_key: Optional[str] = None

    invoice_number: Optional[str] = None

    metadata: Optional[Dict[str, str]] = None

    payment_gateway: Optional[str] = None

    payment_method_id: Optional[str] = None

    payment_method_type: Optional[PaymentMethodType] = None

    payment_status: Optional[PaymentStatus] = None

    payment_url: Optional[str] = None

    refunded_at: Optional[str] = None

    save_card_and_make_default: Optional[bool] = None

    succeeded_at: Optional[str] = None

    tenant_id: Optional[str] = None

    track_attempts: Optional[bool] = None

    updated_at: Optional[str] = None

    updated_by: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "amount",
                "attempts",
                "created_at",
                "created_by",
                "currency",
                "destination_id",
                "destination_type",
                "error_message",
                "failed_at",
                "gateway_metadata",
                "gateway_payment_id",
                "gateway_tracking_id",
                "id",
                "idempotency_key",
                "invoice_number",
                "metadata",
                "payment_gateway",
                "payment_method_id",
                "payment_method_type",
                "payment_status",
                "payment_url",
                "refunded_at",
                "save_card_and_make_default",
                "succeeded_at",
                "tenant_id",
                "track_attempts",
                "updated_at",
                "updated_by",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice.types import BaseModel, UNSET_SENTINEL
from flexprice.utils import FieldMetadata, QueryParamMetadata
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ListScheduledTasksRequestTypedDict(TypedDict):
    limit: NotRequired[int]
    r"""Limit"""
    offset: NotRequired[int]
    r"""Offset"""
    connection_id: NotRequired[str]
    r"""Filter by connection ID"""
    entity_type: NotRequired[str]
    r"""Filter by entity type"""
    interval: NotRequired[str]
    r"""Filter by interval"""
    enabled: NotRequired[bool]
    r"""Filter by enabled status"""


class ListScheduledTasksRequest(BaseModel):
    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Limit"""

    offset: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Offset"""

    connection_id: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Filter by connection ID"""

    entity_type: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Filter by entity type"""

    interval: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Filter by interval"""

    enabled: Annotated[
        Optional[bool],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Filter by enabled status"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["limit", "offset", "connection_id", "entity_type", "interval", "enabled"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .dto_customerlookupresult import (
    DtoCustomerLookupResult,
    DtoCustomerLookupResultTypedDict,
)
from .dto_metermatchingresult import (
    DtoMeterMatchingResult,
    DtoMeterMatchingResultTypedDict,
)
from .dto_pricelookupresult import DtoPriceLookupResult, DtoPriceLookupResultTypedDict
from .dto_subscriptionlineitemlookupresult import (
    DtoSubscriptionLineItemLookupResult,
    DtoSubscriptionLineItemLookupResultTypedDict,
)
from .failurepoint import FailurePoint, FailurePointTypedDict
from flexprice.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class DtoDebugTrackerTypedDict(TypedDict):
    customer_lookup: NotRequired[DtoCustomerLookupResultTypedDict]
    failure_point: NotRequired[FailurePointTypedDict]
    meter_matching: NotRequired[DtoMeterMatchingResultTypedDict]
    price_lookup: NotRequired[DtoPriceLookupResultTypedDict]
    subscription_line_item_lookup: NotRequired[
        DtoSubscriptionLineItemLookupResultTypedDict
    ]


class DtoDebugTracker(BaseModel):
    customer_lookup: Optional[DtoCustomerLookupResult] = None

    failure_point: Optional[FailurePoint] = None

    meter_matching: Optional[DtoMeterMatchingResult] = None

    price_lookup: Optional[DtoPriceLookupResult] = None

    subscription_line_item_lookup: Optional[DtoSubscriptionLineItemLookupResult] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "customer_lookup",
                "failure_point",
                "meter_matching",
                "price_lookup",
                "subscription_line_item_lookup",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .debugtrackerstatus import DebugTrackerStatus
from .errors_errorresponse import ErrorsErrorResponse, ErrorsErrorResponseTypedDict
from .github_com_flexprice_flexprice_internal_domain_customer_customer import (
    GithubComFlexpriceFlexpriceInternalDomainCustomerCustomer,
    GithubComFlexpriceFlexpriceInternalDomainCustomerCustomerTypedDict,
)
from flexprice.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class DtoCustomerLookupResultTypedDict(TypedDict):
    customer: NotRequired[
        GithubComFlexpriceFlexpriceInternalDomainCustomerCustomerTypedDict
    ]
    error: NotRequired[ErrorsErrorResponseTypedDict]
    status: NotRequired[DebugTrackerStatus]


class DtoCustomerLookupResult(BaseModel):
    customer: Optional[GithubComFlexpriceFlexpriceInternalDomainCustomerCustomer] = None

    error: Optional[ErrorsErrorResponse] = None

    status: Optional[DebugTrackerStatus] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["customer", "error", "status"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .dto_createinvoicelineitemrequest import (
    DtoCreateInvoiceLineItemRequest,
    DtoCreateInvoiceLineItemRequestTypedDict,
)
from .dto_invoicecoupon import DtoInvoiceCoupon, DtoInvoiceCouponTypedDict
from .dto_invoicelineitemcoupon import (
    DtoInvoiceLineItemCoupon,
    DtoInvoiceLineItemCouponTypedDict,
)
from .dto_taxrateoverride import DtoTaxRateOverride, DtoTaxRateOverrideTypedDict
from .dto_taxrateresponse import DtoTaxRateResponse, DtoTaxRateResponseTypedDict
from .invoicebillingreason import InvoiceBillingReason
from .invoicestatus import InvoiceStatus
from .invoicetype import InvoiceType
from .paymentstatus import PaymentStatus
from flexprice.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, List, Optional
from typing_extensions import NotRequired, TypedDict


class DtoCreateInvoiceRequestTypedDict(TypedDict):
    amount_due: str
    r"""amount_due is the total amount that needs to be paid for this invoice"""
    currency: str
    r"""currency is the three-letter ISO currency code (e.g., USD, EUR) for the invoice"""
    customer_id: str
    r"""customer_id is the unique identifier of the customer this invoice belongs to"""
    subtotal: str
    r"""subtotal is the amount before taxes and discounts are applied"""
    total: str
    r"""total is the total amount of the invoice including taxes and discounts"""
    amount_paid: NotRequired[str]
    r"""amount_paid is the amount that has been paid towards this invoice"""
    billing_period: NotRequired[str]
    r"""billing_period is the period this invoice covers (e.g., \"monthly\", \"yearly\")"""
    billing_reason: NotRequired[InvoiceBillingReason]
    coupons: NotRequired[List[str]]
    r"""coupons"""
    description: NotRequired[str]
    r"""description is an optional text description of the invoice"""
    due_date: NotRequired[str]
    r"""due_date is the date by which payment is expected"""
    idempotency_key: NotRequired[str]
    r"""idempotency_key is an optional key used to prevent duplicate invoice creation"""
    invoice_coupons: NotRequired[List[DtoInvoiceCouponTypedDict]]
    r"""Invoice Coupons"""
    invoice_number: NotRequired[str]
    r"""invoice_number is an optional human-readable identifier for the invoice"""
    invoice_pdf_url: NotRequired[str]
    r"""invoice_pdf_url is the URL where customers can download the PDF version of this invoice"""
    invoice_status: NotRequired[InvoiceStatus]
    invoice_type: NotRequired[InvoiceType]
    line_item_coupons: NotRequired[List[DtoInvoiceLineItemCouponTypedDict]]
    r"""Invoice Line Item Coupons"""
    line_items: NotRequired[List[DtoCreateInvoiceLineItemRequestTypedDict]]
    r"""line_items contains the individual items that make up this invoice"""
    metadata: NotRequired[Dict[str, str]]
    payment_status: NotRequired[PaymentStatus]
    period_end: NotRequired[str]
    r"""period_end is the end date of the billing period"""
    period_start: NotRequired[str]
    r"""period_start is the start date of the billing period"""
    prepared_tax_rates: NotRequired[List[DtoTaxRateResponseTypedDict]]
    r"""prepared_tax_rates contains the tax rates pre-resolved by the caller (e.g., billing service)"""
    subscription_id: NotRequired[str]
    r"""subscription_id is the optional unique identifier of the subscription associated with this invoice"""
    tax_rate_overrides: NotRequired[List[DtoTaxRateOverrideTypedDict]]
    r"""tax_rate_overrides is the tax rate overrides to be applied to the invoice"""
    tax_rates: NotRequired[List[str]]
    r"""tax_rates"""
    total_prepaid_applied: NotRequired[str]
    r"""total_prepaid_applied is the total amount of prepaid applied to this invoice."""


class DtoCreateInvoiceRequest(BaseModel):
    amount_due: str
    r"""amount_due is the total amount that needs to be paid for this invoice"""

    currency: str
    r"""currency is the three-letter ISO currency code (e.g., USD, EUR) for the invoice"""

    customer_id: str
    r"""customer_id is the unique identifier of the customer this invoice belongs to"""

    subtotal: str
    r"""subtotal is the amount before taxes and discounts are applied"""

    total: str
    r"""total is the total amount of the invoice including taxes and discounts"""

    amount_paid: Optional[str] = None
    r"""amount_paid is the amount that has been paid towards this invoice"""

    billing_period: Optional[str] = None
    r"""billing_period is the period this invoice covers (e.g., \"monthly\", \"yearly\")"""

    billing_reason: Optional[InvoiceBillingReason] = None

    coupons: Optional[List[str]] = None
    r"""coupons"""

    description: Optional[str] = None
    r"""description is an optional text description of the invoice"""

    due_date: Optional[str] = None
    r"""due_date is the date by which payment is expected"""

    idempotency_key: Optional[str] = None
    r"""idempotency_key is an optional key used to prevent duplicate invoice creation"""

    invoice_coupons: Optional[List[DtoInvoiceCoupon]] = None
    r"""Invoice Coupons"""

    invoice_number: Optional[str] = None
    r"""invoice_number is an optional human-readable identifier for the invoice"""

    invoice_pdf_url: Optional[str] = None
    r"""invoice_pdf_url is the URL where customers can download the PDF version of this invoice"""

    invoice_status: Optional[InvoiceStatus] = None

    invoice_type: Optional[InvoiceType] = None

    line_item_coupons: Optional[List[DtoInvoiceLineItemCoupon]] = None
    r"""Invoice Line Item Coupons"""

    line_items: Optional[List[DtoCreateInvoiceLineItemRequest]] = None
    r"""line_items contains the individual items that make up this invoice"""

    metadata: Optional[Dict[str, str]] = None

    payment_status: Optional[PaymentStatus] = None

    period_end: Optional[str] = None
    r"""period_end is the end date of the billing period"""

    period_start: Optional[str] = None
    r"""period_start is the start date of the billing period"""

    prepared_tax_rates: Optional[List[DtoTaxRateResponse]] = None
    r"""prepared_tax_rates contains the tax rates pre-resolved by the caller (e.g., billing service)"""

    subscription_id: Optional[str] = None
    r"""subscription_id is the optional unique identifier of the subscription associated with this invoice"""

    tax_rate_overrides: Optional[List[DtoTaxRateOverride]] = None
    r"""tax_rate_overrides is the tax rate overrides to be applied to the invoice"""

    tax_rates: Optional[List[str]] = None
    r"""tax_rates"""

    total_prepaid_applied: Optional[str] = None
    r"""total_prepaid_applied is the total amount of prepaid applied to this invoice."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "amount_paid",
                "billing_period",
                "billing_reason",
                "coupons",
                "description",
                "due_date",
                "idempotency_key",
                "invoice_coupons",
                "invoice_number",
                "invoice_pdf_url",
                "invoice_status",
                "invoice_type",
                "line_item_coupons",
                "line_items",
                "metadata",
                "payment_status",
                "period_end",
                "period_start",
                "prepared_tax_rates",
                "subscription_id",
                "tax_rate_overrides",
                "tax_rates",
                "total_prepaid_applied",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


Provider = Literal[
    "stripe",
    "razorpay",
    "paypal",
]
r"""provider is the integration provider name (e.g., \"stripe\", \"razorpay\")"""


class DtoIntegrationEntityMappingTypedDict(TypedDict):
    r"""Integration entity mapping for external provider systems"""

    id: str
    r"""id is the external entity ID from the provider"""
    provider: Provider
    r"""provider is the integration provider name (e.g., \"stripe\", \"razorpay\")"""


class DtoIntegrationEntityMapping(BaseModel):
    r"""Integration entity mapping for external provider systems"""

    id: str
    r"""id is the external entity ID from the provider"""

    provider: Provider
    r"""provider is the integration provider name (e.g., \"stripe\", \"razorpay\")"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .dto_createentitlementrequest import (
    DtoCreateEntitlementRequest,
    DtoCreateEntitlementRequestTypedDict,
)
from flexprice.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class DtoCreateBulkEntitlementRequestTypedDict(TypedDict):
    items: List[DtoCreateEntitlementRequestTypedDict]


class DtoCreateBulkEntitlementRequest(BaseModel):
    items: List[DtoCreateEntitlementRequest]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .walletstatus import WalletStatus
from flexprice.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Literal, Optional
from typing_extensions import NotRequired, TypedDict


WalletFilterOrder = Literal[
    "asc",
    "desc",
]


class WalletFilterTypedDict(TypedDict):
    alert_enabled: NotRequired[bool]
    expand: NotRequired[str]
    limit: NotRequired[int]
    offset: NotRequired[int]
    order: NotRequired[WalletFilterOrder]
    sort: NotRequired[str]
    status: NotRequired[WalletStatus]
    wallet_ids: NotRequired[List[str]]


class WalletFilter(BaseModel):
    alert_enabled: Optional[bool] = None

    expand: Optional[str] = None

    limit: Optional[int] = None

    offset: Optional[int] = None

    order: Optional[WalletFilterOrder] = None

    sort: Optional[str] = None

    status: Optional[WalletStatus] = None

    wallet_ids: Optional[List[str]] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "alert_enabled",
                "expand",
                "limit",
                "offset",
                "order",
                "sort",
                "status",
                "wallet_ids",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .creditnotereason import CreditNoteReason
from .dto_createcreditnotelineitemrequest import (
    DtoCreateCreditNoteLineItemRequest,
    DtoCreateCreditNoteLineItemRequestTypedDict,
)
from flexprice.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, List, Optional
from typing_extensions import NotRequired, TypedDict


class DtoCreateCreditNoteRequestTypedDict(TypedDict):
    invoice_id: str
    r"""invoice_id is the unique identifier of the invoice this credit note is applied to"""
    reason: CreditNoteReason
    credit_note_number: NotRequired[str]
    r"""credit_note_number is an optional human-readable identifier for the credit note"""
    idempotency_key: NotRequired[str]
    r"""idempotency_key is an optional key used to prevent duplicate credit note creation"""
    line_items: NotRequired[List[DtoCreateCreditNoteLineItemRequestTypedDict]]
    r"""line_items contains the individual line items that make up this credit note (minimum 1 required)"""
    memo: NotRequired[str]
    r"""memo is an optional free-text field for additional notes about the credit note"""
    metadata: NotRequired[Dict[str, str]]
    process_credit_note: NotRequired[bool]
    r"""process_credit_note is a flag to process the credit note after creation"""


class DtoCreateCreditNoteRequest(BaseModel):
    invoice_id: str
    r"""invoice_id is the unique identifier of the invoice this credit note is applied to"""

    reason: CreditNoteReason

    credit_note_number: Optional[str] = None
    r"""credit_note_number is an optional human-readable identifier for the credit note"""

    idempotency_key: Optional[str] = None
    r"""idempotency_key is an optional key used to prevent duplicate credit note creation"""

    line_items: Optional[List[DtoCreateCreditNoteLineItemRequest]] = None
    r"""line_items contains the individual line items that make up this credit note (minimum 1 required)"""

    memo: Optional[str] = None
    r"""memo is an optional free-text field for additional notes about the credit note"""

    metadata: Optional[Dict[str, str]] = None

    process_credit_note: Optional[bool] = True
    r"""process_credit_note is a flag to process the credit note after creation"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "credit_note_number",
                "idempotency_key",
                "line_items",
                "memo",
                "metadata",
                "process_credit_note",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from flexprice import models, utils
from flexprice._hooks import HookContext
from flexprice.types import OptionalNullable, UNSET
from flexprice.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Dict, List, Mapping, Optional


class Payments(BaseSDK):
    def list_payments(
        self,
        *,
        currency: Optional[str] = None,
        destination_id: Optional[str] = None,
        destination_type: Optional[str] = None,
        end_time: Optional[str] = None,
        expand: Optional[str] = None,
        gateway_payment_id: Optional[str] = None,
        gateway_tracking_id: Optional[str] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        order: Optional[models.ListPaymentsOrder] = None,
        payment_gateway: Optional[str] = None,
        payment_ids: Optional[List[str]] = None,
        payment_method_type: Optional[str] = None,
        payment_status: Optional[str] = None,
        sort: Optional[str] = None,
        start_time: Optional[str] = None,
        status: Optional[models.ListPaymentsStatus] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoListPaymentsResponse:
        r"""List payments

        Use when listing or searching payments (e.g. reconciliation UI or customer payment history). Returns a paginated list; supports filtering by customer, invoice, status.

        :param currency:
        :param destination_id:
        :param destination_type:
        :param end_time:
        :param expand:
        :param gateway_payment_id:
        :param gateway_tracking_id: For filtering by gateway tracking ID
        :param limit:
        :param offset:
        :param order:
        :param payment_gateway:
        :param payment_ids:
        :param payment_method_type:
        :param payment_status:
        :param sort:
        :param start_time:
        :param status:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ListPaymentsRequest(
            currency=currency,
            destination_id=destination_id,
            destination_type=destination_type,
            end_time=end_time,
            expand=expand,
            gateway_payment_id=gateway_payment_id,
            gateway_tracking_id=gateway_tracking_id,
            limit=limit,
            offset=offset,
            order=order,
            payment_gateway=payment_gateway,
            payment_ids=payment_ids,
            payment_method_type=payment_method_type,
            payment_status=payment_status,
            sort=sort,
            start_time=start_time,
            status=status,
        )

        req = self._build_request(
            method="GET",
            path="/payments",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listPayments",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.DtoListPaymentsResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )

    async def list_payments_async(
        self,
        *,
        currency: Optional[str] = None,
        destination_id: Optional[str] = None,
        destination_type: Optional[str] = None,
        end_time: Optional[str] = None,
        expand: Optional[str] = None,
        gateway_payment_id: Optional[str] = None,
        gateway_tracking_id: Optional[str] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        order: Optional[models.ListPaymentsOrder] = None,
        payment_gateway: Optional[str] = None,
        payment_ids: Optional[List[str]] = None,
        payment_method_type: Optional[str] = None,
        payment_status: Optional[str] = None,
        sort: Optional[str] = None,
        start_time: Optional[str] = None,
        status: Optional[models.ListPaymentsStatus] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoListPaymentsResponse:
        r"""List payments

        Use when listing or searching payments (e.g. reconciliation UI or customer payment history). Returns a paginated list; supports filtering by customer, invoice, status.

        :param currency:
        :param destination_id:
        :param destination_type:
        :param end_time:
        :param expand:
        :param gateway_payment_id:
        :param gateway_tracking_id: For filtering by gateway tracking ID
        :param limit:
        :param offset:
        :param order:
        :param payment_gateway:
        :param payment_ids:
        :param payment_method_type:
        :param payment_status:
        :param sort:
        :param start_time:
        :param status:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ListPaymentsRequest(
            currency=currency,
            destination_id=destination_id,
            destination_type=destination_type,
            end_time=end_time,
            expand=expand,
            gateway_payment_id=gateway_payment_id,
            gateway_tracking_id=gateway_tracking_id,
            limit=limit,
            offset=offset,
            order=order,
            payment_gateway=payment_gateway,
            payment_ids=payment_ids,
            payment_method_type=payment_method_type,
            payment_status=payment_status,
            sort=sort,
            start_time=start_time,
            status=status,
        )

        req = self._build_request_async(
            method="GET",
            path="/payments",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listPayments",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.DtoListPaymentsResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )

    def create_payment(
        self,
        *,
        amount: str,
        currency: str,
        destination_id: str,
        destination_type: models.PaymentDestinationType,
        payment_method_type: models.PaymentMethodType,
        cancel_url: Optional[str] = None,
        idempotency_key: Optional[str] = None,
        metadata: Optional[Dict[str, str]] = None,
        payment_gateway: Optional[models.PaymentGatewayType] = None,
        payment_method_id: Optional[str] = None,
        process_payment: Optional[bool] = True,
        save_card_and_make_default: Optional[bool] = False,
        success_url: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoPaymentResponse:
        r"""Create payment

        Use when recording a payment against an invoice (e.g. after receiving funds via a gateway or manual entry).

        :param amount:
        :param currency:
        :param destination_id:
        :param destination_type:
        :param payment_method_type:
        :param cancel_url:
        :param idempotency_key:
        :param metadata:
        :param payment_gateway:
        :param payment_method_id:
        :param process_payment:
        :param save_card_and_make_default:
        :param success_url:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.DtoCreatePaymentRequest(
            amount=amount,
            cancel_url=cancel_url,
            currency=currency,
            destination_id=destination_id,
            destination_type=destination_type,
            idempotency_key=idempotency_key,
            metadata=metadata,
            payment_gateway=payment_gateway,
            payment_method_id=payment_method_id,
            payment_method_type=payment_method_type,
            process_payment=process_payment,
            save_card_and_make_default=save_card_and_make_default,
            success_url=success_url,
        )

        req = self._build_request(
            method="POST",
            path="/payments",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.DtoCreatePaymentRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createPayment",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(models.DtoPaymentResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )

    async def create_payment_async(
        self,
        *,
        amount: str,
        currency: str,
        destination_id: str,
        destination_type: models.PaymentDestinationType,
        payment_method_type: models.PaymentMethodType,
        cancel_url: Optional[str] = None,
        idempotency_key: Optional[str] = None,
        metadata: Optional[Dict[str, str]] = None,
        payment_gateway: Optional[models.PaymentGatewayType] = None,
        payment_method_id: Optional[str] = None,
        process_payment: Optional[bool] = True,
        save_card_and_make_default: Optional[bool] = False,
        success_url: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoPaymentResponse:
        r"""Create payment

        Use when recording a payment against an invoice (e.g. after receiving funds via a gateway or manual entry).

        :param amount:
        :param currency:
        :param destination_id:
        :param destination_type:
        :param payment_method_type:
        :param cancel_url:
        :param idempotency_key:
        :param metadata:
        :param payment_gateway:
        :param payment_method_id:
        :param process_payment:
        :param save_card_and_make_default:
        :param success_url:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.DtoCreatePaymentRequest(
            amount=amount,
            cancel_url=cancel_url,
            currency=currency,
            destination_id=destination_id,
            destination_type=destination_type,
            idempotency_key=idempotency_key,
            metadata=metadata,
            payment_gateway=payment_gateway,
            payment_method_id=payment_method_id,
            payment_method_type=payment_method_type,
            process_payment=process_payment,
            save_card_and_make_default=save_card_and_make_default,
            success_url=success_url,
        )

        req = self._build_request_async(
            method="POST",
            path="/payments",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.DtoCreatePaymentRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createPayment",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(models.DtoPaymentResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )

    def get_payment(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoPaymentResponse:
        r"""Get payment

        Use when you need to load a single payment (e.g. for a receipt view or reconciliation).

        :param id: Payment ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetPaymentRequest(
            id=id,
        )

        req = self._build_request(
            method="GET",
            path="/payments/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getPayment",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.DtoPaymentResponse, http_res)
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )

    async def get_payment_async(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoPaymentResponse:
        r"""Get payment

        Use when you need to load a single payment (e.g. for a receipt view or reconciliation).

        :param id: Payment ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetPaymentRequest(
            id=id,
        )

        req = self._build_request_async(
            method="GET",
            path="/payments/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getPayment",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.DtoPaymentResponse, http_res)
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )

    def update_payment(
        self,
        *,
        id: str,
        error_message: Optional[str] = None,
        failed_at: Optional[str] = None,
        gateway_payment_id: Optional[str] = None,
        metadata: Optional[Dict[str, str]] = None,
        payment_gateway: Optional[str] = None,
        payment_method_id: Optional[str] = None,
        payment_status: Optional[str] = None,
        succeeded_at: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoPaymentResponse:
        r"""Update payment

        Use when updating payment status or metadata (e.g. after reconciliation or adding a reference).

        :param id: Payment ID
        :param error_message:
        :param failed_at:
        :param gateway_payment_id:
        :param metadata:
        :param payment_gateway:
        :param payment_method_id:
        :param payment_status:
        :param succeeded_at:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.UpdatePaymentRequest(
            id=id,
            body=models.DtoUpdatePaymentRequest(
                error_message=error_message,
                failed_at=failed_at,
                gateway_payment_id=gateway_payment_id,
                metadata=metadata,
                payment_gateway=payment_gateway,
                payment_method_id=payment_method_id,
                payment_status=payment_status,
                succeeded_at=succeeded_at,
            ),
        )

        req = self._build_request(
            method="PUT",
            path="/payments/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body, False, False, "json", models.DtoUpdatePaymentRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="updatePayment",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.DtoPaymentResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )

    async def update_payment_async(
        self,
        *,
        id: str,
        error_message: Optional[str] = None,
        failed_at: Optional[str] = None,
        gateway_payment_id: Optional[str] = None,
        metadata: Optional[Dict[str, str]] = None,
        payment_gateway: Optional[str] = None,
        payment_method_id: Optional[str] = None,
        payment_status: Optional[str] = None,
        succeeded_at: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoPaymentResponse:
        r"""Update payment

        Use when updating payment status or metadata (e.g. after reconciliation or adding a reference).

        :param id: Payment ID
        :param error_message:
        :param failed_at:
        :param gateway_payment_id:
        :param metadata:
        :param payment_gateway:
        :param payment_method_id:
        :param payment_status:
        :param succeeded_at:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.UpdatePaymentRequest(
            id=id,
            body=models.DtoUpdatePaymentRequest(
                error_message=error_message,
                failed_at=failed_at,
                gateway_payment_id=gateway_payment_id,
                metadata=metadata,
                payment_gateway=payment_gateway,
                payment_method_id=payment_method_id,
                payment_status=payment_status,
                succeeded_at=succeeded_at,
            ),
        )

        req = self._build_request_async(
            method="PUT",
            path="/payments/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body, False, False, "json", models.DtoUpdatePaymentRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="updatePayment",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.DtoPaymentResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )

    def delete_payment(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoSuccessResponse:
        r"""Delete payment

        Use when removing or voiding a payment record (e.g. correcting erroneous entries). Returns 200 with success message.

        :param id: Payment ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.DeletePaymentRequest(
            id=id,
        )

        req = self._build_request(
            method="DELETE",
            path="/payments/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="deletePayment",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.DtoSuccessResponse, http_res)
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )

    async def delete_payment_async(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoSuccessResponse:
        r"""Delete payment

        Use when removing or voiding a payment record (e.g. correcting erroneous entries). Returns 200 with success message.

        :param id: Payment ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.DeletePaymentRequest(
            id=id,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/payments/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="deletePayment",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.DtoSuccessResponse, http_res)
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )

    def process_payment(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoPaymentResponse:
        r"""Process payment

        Use when you need to charge or process a payment (e.g. trigger the payment provider to capture funds). Returns updated payment with status.

        :param id: Payment ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ProcessPaymentRequest(
            id=id,
        )

        req = self._build_request(
            method="POST",
            path="/payments/{id}/process",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="processPayment",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.DtoPaymentResponse, http_res)
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )

    async def process_payment_async(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoPaymentResponse:
        r"""Process payment

        Use when you need to charge or process a payment (e.g. trigger the payment provider to capture funds). Returns updated payment with status.

        :param id: Payment ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ProcessPaymentRequest(
            id=id,
        )

        req = self._build_request_async(
            method="POST",
            path="/payments/{id}/process",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="processPayment",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.DtoPaymentResponse, http_res)
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice.types import UnrecognizedStr
from typing import Literal, Union


BillingCycle = Union[
    Literal[
        "anniversary",
        "calendar",
    ],
    UnrecognizedStr,
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .filtercondition import FilterCondition, FilterConditionTypedDict
from .queryfilter import QueryFilter, QueryFilterTypedDict
from .sortcondition import SortCondition, SortConditionTypedDict
from .status import Status
from .timerangefilter import TimeRangeFilter, TimeRangeFilterTypedDict
from flexprice.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CostsheetFilterTypedDict(TypedDict):
    costsheet_i_ds: NotRequired[List[str]]
    r"""CostsheetIDs allows filtering by specific costsheet IDs"""
    environment_id: NotRequired[str]
    r"""EnvironmentID filters by specific environment ID"""
    filters: NotRequired[List[FilterConditionTypedDict]]
    r"""Filters contains custom filtering conditions"""
    lookup_key: NotRequired[str]
    r"""LookupKey filters by lookup key"""
    name: NotRequired[str]
    r"""Name filters by costsheet name"""
    query_filter: NotRequired[QueryFilterTypedDict]
    sort: NotRequired[List[SortConditionTypedDict]]
    r"""Sort specifies result ordering preferences"""
    status: NotRequired[Status]
    tenant_id: NotRequired[str]
    r"""TenantID filters by specific tenant ID"""
    time_range_filter: NotRequired[TimeRangeFilterTypedDict]


class CostsheetFilter(BaseModel):
    costsheet_i_ds: Annotated[
        Optional[List[str]], pydantic.Field(alias="costsheetIDs")
    ] = None
    r"""CostsheetIDs allows filtering by specific costsheet IDs"""

    environment_id: Annotated[Optional[str], pydantic.Field(alias="environmentID")] = (
        None
    )
    r"""EnvironmentID filters by specific environment ID"""

    filters: Optional[List[FilterCondition]] = None
    r"""Filters contains custom filtering conditions"""

    lookup_key: Annotated[Optional[str], pydantic.Field(alias="lookupKey")] = None
    r"""LookupKey filters by lookup key"""

    name: Optional[str] = None
    r"""Name filters by costsheet name"""

    query_filter: Annotated[
        Optional[QueryFilter], pydantic.Field(alias="queryFilter")
    ] = None

    sort: Optional[List[SortCondition]] = None
    r"""Sort specifies result ordering preferences"""

    status: Optional[Status] = None

    tenant_id: Annotated[Optional[str], pydantic.Field(alias="tenantID")] = None
    r"""TenantID filters by specific tenant ID"""

    time_range_filter: Annotated[
        Optional[TimeRangeFilter], pydantic.Field(alias="timeRangeFilter")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "costsheetIDs",
                "environmentID",
                "filters",
                "lookupKey",
                "name",
                "queryFilter",
                "sort",
                "status",
                "tenantID",
                "timeRangeFilter",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    CostsheetFilter.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from flexprice import models, utils
from flexprice._hooks import HookContext
from flexprice.types import OptionalNullable, UNSET
from flexprice.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, List, Mapping, Optional, Union


class Users(BaseSDK):
    def create_user(
        self,
        *,
        type_: models.UserType,
        email: Optional[str] = None,
        roles: Optional[List[str]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoCreateUserResponse:
        r"""Create user or service account

        Create a user account (type=user, email required; returns user + password for login) or a service account (type=service_account, roles required) for API/automation access.

        :param type:
        :param email: Required when type is \"user\"
        :param roles: Required when type is \"service_account\"
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.DtoCreateUserRequest(
            email=email,
            roles=roles,
            type=type_,
        )

        req = self._build_request(
            method="POST",
            path="/users",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.DtoCreateUserRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createUser",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(models.DtoCreateUserResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )

    async def create_user_async(
        self,
        *,
        type_: models.UserType,
        email: Optional[str] = None,
        roles: Optional[List[str]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoCreateUserResponse:
        r"""Create user or service account

        Create a user account (type=user, email required; returns user + password for login) or a service account (type=service_account, roles required) for API/automation access.

        :param type:
        :param email: Required when type is \"user\"
        :param roles: Required when type is \"service_account\"
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.DtoCreateUserRequest(
            email=email,
            roles=roles,
            type=type_,
        )

        req = self._build_request_async(
            method="POST",
            path="/users",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.DtoCreateUserRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createUser",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(models.DtoCreateUserResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )

    def get_user_info(
        self,
        *,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoUserResponse:
        r"""Get current user

        Use to show the logged-in user's profile in the UI or to check permissions and roles for the current session.

        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)
        req = self._build_request(
            method="GET",
            path="/users/me",
            base_url=base_url,
            url_variables=url_variables,
            request=None,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getUserInfo",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["401", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.DtoUserResponse, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )

    async def get_user_info_async(
        self,
        *,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoUserResponse:
        r"""Get current user

        Use to show the logged-in user's profile in the UI or to check permissions and roles for the current session.

        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)
        req = self._build_request_async(
            method="GET",
            path="/users/me",
            base_url=base_url,
            url_variables=url_variables,
            request=None,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getUserInfo",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["401", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.DtoUserResponse, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )

    def query_user(
        self,
        *,
        end_time: Optional[str] = None,
        expand: Optional[str] = None,
        filters: Optional[
            Union[List[models.FilterCondition], List[models.FilterConditionTypedDict]]
        ] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        order: Optional[models.UserFilterOrder] = None,
        roles: Optional[List[str]] = None,
        sort: Optional[
            Union[List[models.SortCondition], List[models.SortConditionTypedDict]]
        ] = None,
        start_time: Optional[str] = None,
        status: Optional[models.Status] = None,
        type_: Optional[models.UserType] = None,
        user_ids: Optional[List[str]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoListUsersResponse:
        r"""Query users

        Use when listing or searching service accounts in an admin UI, or when auditing who has API access and which roles they have.

        :param end_time:
        :param expand:
        :param filters: filters allows complex filtering based on multiple fields
        :param limit:
        :param offset:
        :param order:
        :param roles:
        :param sort:
        :param start_time:
        :param status:
        :param type:
        :param user_ids: Specific filters for users
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.UserFilter(
            end_time=end_time,
            expand=expand,
            filters=utils.get_pydantic_model(
                filters, Optional[List[models.FilterCondition]]
            ),
            limit=limit,
            offset=offset,
            order=order,
            roles=roles,
            sort=utils.get_pydantic_model(sort, Optional[List[models.SortCondition]]),
            start_time=start_time,
            status=status,
            type=type_,
            user_ids=user_ids,
        )

        req = self._build_request(
            method="POST",
            path="/users/search",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.UserFilter
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="queryUser",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.DtoListUsersResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )

    async def query_user_async(
        self,
        *,
        end_time: Optional[str] = None,
        expand: Optional[str] = None,
        filters: Optional[
            Union[List[models.FilterCondition], List[models.FilterConditionTypedDict]]
        ] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        order: Optional[models.UserFilterOrder] = None,
        roles: Optional[List[str]] = None,
        sort: Optional[
            Union[List[models.SortCondition], List[models.SortConditionTypedDict]]
        ] = None,
        start_time: Optional[str] = None,
        status: Optional[models.Status] = None,
        type_: Optional[models.UserType] = None,
        user_ids: Optional[List[str]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DtoListUsersResponse:
        r"""Query users

        Use when listing or searching service accounts in an admin UI, or when auditing who has API access and which roles they have.

        :param end_time:
        :param expand:
        :param filters: filters allows complex filtering based on multiple fields
        :param limit:
        :param offset:
        :param order:
        :param roles:
        :param sort:
        :param start_time:
        :param status:
        :param type:
        :param user_ids: Specific filters for users
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.UserFilter(
            end_time=end_time,
            expand=expand,
            filters=utils.get_pydantic_model(
                filters, Optional[List[models.FilterCondition]]
            ),
            limit=limit,
            offset=offset,
            order=order,
            roles=roles,
            sort=utils.get_pydantic_model(sort, Optional[List[models.SortCondition]]),
            start_time=start_time,
            status=status,
            type=type_,
            user_ids=user_ids,
        )

        req = self._build_request_async(
            method="POST",
            path="/users/search",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.UserFilter
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="queryUser",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.DtoListUsersResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.errors.ErrorsErrorResponseData, http_res
            )
            raise models.errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise models.errors.FlexpriceDefaultError(
            "Unexpected response received", http_res
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .aggregationtype import AggregationType
from .windowsize import WindowSize
from flexprice.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, List, Optional
from typing_extensions import NotRequired, TypedDict


class DtoGetUsageRequestTypedDict(TypedDict):
    aggregation_type: AggregationType
    event_name: str
    billing_anchor: NotRequired[str]
    r"""BillingAnchor enables custom monthly billing periods for usage aggregation.

    When to use:
    - WindowSize = \"MONTH\" AND you need custom monthly periods (not calendar months)
    - Subscription billing that doesn't align with calendar months
    - Example: Customer signed up on 15th, so billing periods are 15th to 15th

    When NOT to use:
    - WindowSize != \"MONTH\" (ignored for DAY, HOUR, WEEK, etc.)
    - Standard calendar-based billing (1st to 1st of each month)

    Example values:
    - \"2024-03-05T14:30:45.123456789Z\" (5th of each month at 2:30:45 PM)
    - \"2024-01-15T00:00:00Z\" (15th of each month at midnight)
    - \"2024-02-29T12:00:00Z\" (29th of each month at noon - handles leap years)
    """
    bucket_size: NotRequired[WindowSize]
    customer_id: NotRequired[str]
    end_time: NotRequired[str]
    external_customer_id: NotRequired[str]
    filters: NotRequired[Dict[str, List[str]]]
    group_by_property: NotRequired[str]
    r"""GroupByProperty is the property name in event.properties to group by before aggregating.
    When set, aggregation is applied per unique value of this property within each bucket,
    then the per-group results are summed to produce the bucket total.
    """
    multiplier: NotRequired[str]
    property_name: NotRequired[str]
    r"""will be empty/ignored in case of COUNT"""
    start_time: NotRequired[str]
    window_size: NotRequired[WindowSize]


class DtoGetUsageRequest(BaseModel):
    aggregation_type: AggregationType

    event_name: str

    billing_anchor: Optional[str] = None
    r"""BillingAnchor enables custom monthly billing periods for usage aggregation.

    When to use:
    - WindowSize = \"MONTH\" AND you need custom monthly periods (not calendar months)
    - Subscription billing that doesn't align with calendar months
    - Example: Customer signed up on 15th, so billing periods are 15th to 15th

    When NOT to use:
    - WindowSize != \"MONTH\" (ignored for DAY, HOUR, WEEK, etc.)
    - Standard calendar-based billing (1st to 1st of each month)

    Example values:
    - \"2024-03-05T14:30:45.123456789Z\" (5th of each month at 2:30:45 PM)
    - \"2024-01-15T00:00:00Z\" (15th of each month at midnight)
    - \"2024-02-29T12:00:00Z\" (29th of each month at noon - handles leap years)
    """

    bucket_size: Optional[WindowSize] = None

    customer_id: Optional[str] = None

    end_time: Optional[str] = None

    external_customer_id: Optional[str] = None

    filters: Optional[Dict[str, List[str]]] = None

    group_by_property: Optional[str] = None
    r"""GroupByProperty is the property name in event.properties to group by before aggregating.
    When set, aggregation is applied per unique value of this property within each bucket,
    then the per-group results are summed to produce the bucket total.
    """

    multiplier: Optional[str] = None

    property_name: Optional[str] = None
    r"""will be empty/ignored in case of COUNT"""

    start_time: Optional[str] = None

    window_size: Optional[WindowSize] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "billing_anchor",
                "bucket_size",
                "customer_id",
                "end_time",
                "external_customer_id",
                "filters",
                "group_by_property",
                "multiplier",
                "property_name",
                "start_time",
                "window_size",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .dto_creditgrantresponse import (
    DtoCreditGrantResponse,
    DtoCreditGrantResponseTypedDict,
)
from .dto_invoiceresponse import DtoInvoiceResponse, DtoInvoiceResponseTypedDict
from .dto_prorationdetails import DtoProrationDetails, DtoProrationDetailsTypedDict
from .dto_subscriptionsummary import (
    DtoSubscriptionSummary,
    DtoSubscriptionSummaryTypedDict,
)
from .subscriptionchangetype import SubscriptionChangeType
from flexprice.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, List, Optional
from typing_extensions import NotRequired, TypedDict


class DtoSubscriptionChangeExecuteResponseTypedDict(TypedDict):
    r"""Response after successfully executing a subscription plan change"""

    change_type: NotRequired[SubscriptionChangeType]
    credit_grants: NotRequired[List[DtoCreditGrantResponseTypedDict]]
    r"""credit_grants contains any credit grants created for proration credits"""
    effective_date: NotRequired[str]
    r"""effective_date is when the change took effect"""
    invoice: NotRequired[DtoInvoiceResponseTypedDict]
    is_scheduled: NotRequired[bool]
    r"""is_scheduled indicates if the change was scheduled or executed immediately"""
    metadata: NotRequired[Dict[str, str]]
    r"""metadata from the request"""
    new_subscription: NotRequired[DtoSubscriptionSummaryTypedDict]
    old_subscription: NotRequired[DtoSubscriptionSummaryTypedDict]
    proration_applied: NotRequired[DtoProrationDetailsTypedDict]
    schedule_id: NotRequired[str]
    r"""schedule_id is the ID of the created schedule (only if is_scheduled=true)"""
    scheduled_at: NotRequired[str]
    r"""scheduled_at is when the change will execute (only if is_scheduled=true)"""


class DtoSubscriptionChangeExecuteResponse(BaseModel):
    r"""Response after successfully executing a subscription plan change"""

    change_type: Optional[SubscriptionChangeType] = None

    credit_grants: Optional[List[DtoCreditGrantResponse]] = None
    r"""credit_grants contains any credit grants created for proration credits"""

    effective_date: Optional[str] = None
    r"""effective_date is when the change took effect"""

    invoice: Optional[DtoInvoiceResponse] = None

    is_scheduled: Optional[bool] = None
    r"""is_scheduled indicates if the change was scheduled or executed immediately"""

    metadata: Optional[Dict[str, str]] = None
    r"""metadata from the request"""

    new_subscription: Optional[DtoSubscriptionSummary] = None

    old_subscription: Optional[DtoSubscriptionSummary] = None

    proration_applied: Optional[DtoProrationDetails] = None

    schedule_id: Optional[str] = None
    r"""schedule_id is the ID of the created schedule (only if is_scheduled=true)"""

    scheduled_at: Optional[str] = None
    r"""scheduled_at is when the change will execute (only if is_scheduled=true)"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "change_type",
                "credit_grants",
                "effective_date",
                "invoice",
                "is_scheduled",
                "metadata",
                "new_subscription",
                "old_subscription",
                "proration_applied",
                "schedule_id",
                "scheduled_at",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .httpclient import AsyncHttpClient, ClientOwner, HttpClient, close_clients
from .sdkconfiguration import SDKConfiguration
from .utils.logger import Logger, get_default_logger
from .utils.retries import RetryConfig
from flexprice import models, utils
from flexprice._hooks import SDKHooks
from flexprice.types import OptionalNullable, UNSET
import httpx
import importlib
import sys
from typing import Any, Callable, Dict, Optional, TYPE_CHECKING, Union, cast
import weakref

if TYPE_CHECKING:
    from flexprice.addons import Addons
    from flexprice.alerts import Alerts
    from flexprice.costs import Costs
    from flexprice.coupons import Coupons
    from flexprice.credit_grants import CreditGrants
    from flexprice.credit_notes import CreditNotes
    from flexprice.customers import Customers
    from flexprice.entitlements import Entitlements
    from flexprice.entity_integration_mappings import EntityIntegrationMappings
    from flexprice.events import Events
    from flexprice.features import Features
    from flexprice.groups import Groups
    from flexprice.integrations import Integrations
    from flexprice.invoices import Invoices
    from flexprice.payments import Payments
    from flexprice.plans import Plans
    from flexprice.price_units import PriceUnits
    from flexprice.prices import Prices
    from flexprice.rbac import Rbac
    from flexprice.scheduled_tasks import ScheduledTasks
    from flexprice.secrets import Secrets
    from flexprice.subscriptions import Subscriptions
    from flexprice.tasks import Tasks
    from flexprice.tax_associations import TaxAssociations
    from flexprice.tax_rates import TaxRates
    from flexprice.tenants import Tenants
    from flexprice.users import Users
    from flexprice.wallets import Wallets
    from flexprice.webhooks import Webhooks
    from flexprice.workflows import Workflows


class Flexprice(BaseSDK):
    r"""Flexprice API: Flexprice API Service"""

    addons: "Addons"
    entitlements: "Entitlements"
    alerts: "Alerts"
    costs: "Costs"
    coupons: "Coupons"
    credit_grants: "CreditGrants"
    credit_notes: "CreditNotes"
    customers: "Customers"
    wallets: "Wallets"
    invoices: "Invoices"
    entity_integration_mappings: "EntityIntegrationMappings"
    events: "Events"
    features: "Features"
    groups: "Groups"
    payments: "Payments"
    plans: "Plans"
    prices: "Prices"
    price_units: "PriceUnits"
    rbac: "Rbac"
    secrets: "Secrets"
    integrations: "Integrations"
    subscriptions: "Subscriptions"
    tasks: "Tasks"
    scheduled_tasks: "ScheduledTasks"
    tax_associations: "TaxAssociations"
    tax_rates: "TaxRates"
    tenants: "Tenants"
    users: "Users"
    webhooks: "Webhooks"
    workflows: "Workflows"
    _sub_sdk_map = {
        "addons": ("flexprice.addons", "Addons"),
        "entitlements": ("flexprice.entitlements", "Entitlements"),
        "alerts": ("flexprice.alerts", "Alerts"),
        "costs": ("flexprice.costs", "Costs"),
        "coupons": ("flexprice.coupons", "Coupons"),
        "credit_grants": ("flexprice.credit_grants", "CreditGrants"),
        "credit_notes": ("flexprice.credit_notes", "CreditNotes"),
        "customers": ("flexprice.customers", "Customers"),
        "wallets": ("flexprice.wallets", "Wallets"),
        "invoices": ("flexprice.invoices", "Invoices"),
        "entity_integration_mappings": (
            "flexprice.entity_integration_mappings",
            "EntityIntegrationMappings",
        ),
        "events": ("flexprice.events", "Events"),
        "features": ("flexprice.features", "Features"),
        "groups": ("flexprice.groups", "Groups"),
        "payments": ("flexprice.payments", "Payments"),
        "plans": ("flexprice.plans", "Plans"),
        "prices": ("flexprice.prices", "Prices"),
        "price_units": ("flexprice.price_units", "PriceUnits"),
        "rbac": ("flexprice.rbac", "Rbac"),
        "secrets": ("flexprice.secrets", "Secrets"),
        "integrations": ("flexprice.integrations", "Integrations"),
        "subscriptions": ("flexprice.subscriptions", "Subscriptions"),
        "tasks": ("flexprice.tasks", "Tasks"),
        "scheduled_tasks": ("flexprice.scheduled_tasks", "ScheduledTasks"),
        "tax_associations": ("flexprice.tax_associations", "TaxAssociations"),
        "tax_rates": ("flexprice.tax_rates", "TaxRates"),
        "tenants": ("flexprice.tenants", "Tenants"),
        "users": ("flexprice.users", "Users"),
        "webhooks": ("flexprice.webhooks", "Webhooks"),
        "workflows": ("flexprice.workflows", "Workflows"),
    }

    def __init__(
        self,
        api_key_auth: Union[str, Callable[[], str]],
        server_idx: Optional[int] = None,
        url_params: Optional[Dict[str, str]] = None,
        server_url: Optional[str] = None,
        client: Optional[HttpClient] = None,
        async_client: Optional[AsyncHttpClient] = None,
        retry_config: OptionalNullable[RetryConfig] = UNSET,
        timeout_ms: Optional[int] = None,
        debug_logger: Optional[Logger] = None,
    ) -> None:
        r"""Instantiates the SDK configuring it with the provided parameters.

        :param api_key_auth: The api_key_auth required for authentication
        :param server_idx: The index of the server to use for all methods
        :param server_url: The server URL to use for all methods
        :param url_params: Parameters to optionally template the server URL with
        :param client: The HTTP client to use for all synchronous methods
        :param async_client: The Async HTTP client to use for all asynchronous methods
        :param retry_config: The retry configuration to use for all supported methods
        :param timeout_ms: Optional request timeout applied to each operation in milliseconds
        """
        client_supplied = True
        if client is None:
            client = httpx.Client(follow_redirects=True)
            client_supplied = False

        assert issubclass(
            type(client), HttpClient
        ), "The provided client must implement the HttpClient protocol."

        async_client_supplied = True
        if async_client is None:
            async_client = httpx.AsyncClient(follow_redirects=True)
            async_client_supplied = False

        if debug_logger is None:
            debug_logger = get_default_logger()

        assert issubclass(
            type(async_client), AsyncHttpClient
        ), "The provided async_client must implement the AsyncHttpClient protocol."

        security: Any = None
        if callable(api_key_auth):
            # pylint: disable=unnecessary-lambda-assignment
            security = lambda: models.Security(api_key_auth=api_key_auth())
        else:
            security = models.Security(api_key_auth=api_key_auth)

        if server_url is not None:
            if url_params is not None:
                server_url = utils.template_url(server_url, url_params)

        BaseSDK.__init__(
            self,
            SDKConfiguration(
                client=client,
                client_supplied=client_supplied,
                async_client=async_client,
                async_client_supplied=async_client_supplied,
                security=security,
                server_url=server_url,
                server_idx=server_idx,
                retry_config=retry_config,
                timeout_ms=timeout_ms,
                debug_logger=debug_logger,
            ),
            parent_ref=self,
        )

        hooks = SDKHooks()

        # pylint: disable=protected-access
        self.sdk_configuration.__dict__["_hooks"] = hooks

        self.sdk_configuration = hooks.sdk_init(self.sdk_configuration)

        weakref.finalize(
            self,
            close_clients,
            cast(ClientOwner, self.sdk_configuration),
            self.sdk_configuration.client,
            self.sdk_configuration.client_supplied,
            self.sdk_configuration.async_client,
            self.sdk_configuration.async_client_supplied,
        )

    def dynamic_import(self, modname, retries=3):
        for attempt in range(retries):
            try:
                return importlib.import_module(modname)
            except KeyError:
                # Clear any half-initialized module and retry
                sys.modules.pop(modname, None)
                if attempt == retries - 1:
                    break
        raise KeyError(f"Failed to import module '{modname}' after {retries} attempts")

    def __getattr__(self, name: str):
        if name in self._sub_sdk_map:
            module_path, class_name = self._sub_sdk_map[name]
            try:
                module = self.dynamic_import(module_path)
                klass = getattr(module, class_name)
                instance = klass(self.sdk_configuration, parent_ref=self)
                setattr(self, name, instance)
                return instance
            except ImportError as e:
                raise AttributeError(
                    f"Failed to import module {module_path} for attribute {name}: {e}"
                ) from e
            except AttributeError as e:
                raise AttributeError(
                    f"Failed to find class {class_name} in module {module_path} for attribute {name}: {e}"
                ) from e

        raise AttributeError(
            f"'{type(self).__name__}' object has no attribute '{name}'"
        )

    def __dir__(self):
        default_attrs = list(super().__dir__())
        lazy_attrs = list(self._sub_sdk_map.keys())
        return sorted(list(set(default_attrs + lazy_attrs)))

    def __enter__(self):
        return self

    async def __aenter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if (
            self.sdk_configuration.client is not None
            and not self.sdk_configuration.client_supplied
        ):
            self.sdk_configuration.client.close()
        self.sdk_configuration.client = None

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if (
            self.sdk_configuration.async_client is not None
            and not self.sdk_configuration.async_client_supplied
        ):
            await self.sdk_configuration.async_client.aclose()
        self.sdk_configuration.async_client = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from typing import Literal


PaymentBehavior = Literal[
    "allow_incomplete",
    "default_incomplete",
    "error_if_incomplete",
    "default_active",
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from typing import Literal


InvoiceBilling = Literal[
    "invoice_to_parent",
    "invoice_to_self",
]
